<?php
/**
 * Plugin Name: Wordpress Portfolio Plugin
 * Plugin URI: http://wordpress.org/extend/plugins/wp-portfolio/
 * Description: A plugin that allows you to show off your portfolio through a single page on your wordpress blog with automatically generated thumbnails. To show your portfolio, create a new page and paste &lt;!&#45;&#45; ShowMyPortfolio &#45;&#45;&gt; into it. The plugin requires you to have a free account with <a href="http://www.shrinktheweb.com/">Shrink The Web</a> to generate the thumbnails.
 * Version: 1.03
 * Author: Dan Harrison
 * Author URI: http://www.danharrison.co.uk 
 */
/**
 * This plugin is licensed under the Apache 2 License
 * http://www.apache.org/licenses/LICENSE-2.0
 */

/* Libaries */
if (!defined('DANHARRISON_WPLIB')) {
	include_once('wplib/utils_formbuilder.inc.php');
	include_once('wplib/utils_tablebuilder.inc.php');
	include_once('wplib/utils_sql.inc.php');
	define('DANHARRISON_WPLIB', true);
}


/** Constant: The string used to determine when to render the portfolio. */
define('WPP_STR_SHOW_PORTFOLIO', 				'<!-- ShowMyPortfolio -->');

/** Constant: The string used to determine when to render a group name. */
define('WPP_STR_GROUP_NAME', 					'%GROUP_NAME%');

/** Constant: The string used to determine when to render a group description. */
define('WPP_STR_GROUP_DESCRIPTION', 	 		'%GROUP_DESCRIPTION%');

/** Constant: The string used to determine when to render a website name. */
define('WPP_STR_WEBSITE_NAME', 	 				'%WEBSITE_NAME%');

/** Constant: The string used to determine when to render a website thumbnail image. */
define('WPP_STR_WEBSITE_THUMBNAIL', 	 		'%WEBSITE_THUMBNAIL%');

/** Constant: The string used to determine when to render a website url. */
define('WPP_STR_WEBSITE_URL', 	 				'%WEBSITE_URL%');

/** Constant: The string used to determine when to render a website description. */
define('WPP_STR_WEBSITE_DESCRIPTION', 	 		'%WEBSITE_DESCRIPTION%');

/** Constant: Default HTML to render a group. */
define('WPP_DEFAULT_GROUP_TEMPLATE', 			
"<h2>%GROUP_NAME%</h2>
<p>%GROUP_DESCRIPTION%</p>");

/** Constant: Default HTML to render a website. */
define('WPP_DEFAULT_WEBSITE_TEMPLATE', 			
"<div class=\"portfolio-website\">
    <div class=\"website-thumbnail\"><a href=\"%WEBSITE_URL%\" target=\"_blank\">%WEBSITE_THUMBNAIL%</a></div>
    <div class=\"website-name\"><a href=\"%WEBSITE_URL%\" target=\"_blank\">%WEBSITE_NAME%</a></div>
    <div class=\"website-url\"><a href=\"%WEBSITE_URL%\" target=\"_blank\">%WEBSITE_URL%</a></div>
    <div class=\"website-description\">%WEBSITE_DESCRIPTION%</div>
    <div class=\"website-clear\"></div>
</div>");

/** Constant: Default CSS to style the portfolio. */
define('WPP_DEFAULT_CSS',
".portfolio-website {
	padding: 10px;
	margin-bottom: 10px;
}
.website-thumbnail {
	float: left;
	margin: 0 20px 20px 0;
}
.website-thumbnail img {
	border: 1px solid #555;
	margin: 0;
	padding: 0;
}
.website-name {
	font-size: 12pt;
	font-weight: bold;
	margin-bottom: 3px;
}
.website-name a,.website-url a {
	text-decoration: none;
}
.website-name a:hover,.website-url a:hover {
	text-decoration: underline;
}
.website-url {
	font-size: 9pt;
	font-weight: bold;
}
.website-url a {
	color: #777;
}
.website-description {
	margin-top: 15px;
}
.website-clear {
	clear: both;
}");


/** Constant: The name of the table to store the website information. */
define('TABLE_WEBSITES', 						'WPPortfolio_websites');

/** Constant: The name of the table to store the website information. */
define('TABLE_WEBSITE_GROUPS', 					'WPPortfolio_groups');

/** Contstant: The path to use to store the cached thumbnails. */
define('WPP_THUMBNAIL_PATH',					'wp-portfolio/cache');

/** Contstant: The path to use to store the cached thumbnails. */
define('WPP_THUMB_DEFAULTS',				'wp-portfolio/thumbnail_blank_');

/** Constant: URL location for settings page. */
define('WPP_SETTINGS', 							'admin.php?page=WPP_show_settings');

/** Constant: URL location for website summary. */
define('WPP_WEBSITE_SUMMARY', 					'admin.php?page=wp-portfolio/wp-portfolio.php');

/** Constant: URL location for modifying a website entry. */
define('WPP_MODIFY_WEBSITE', 					'admin.php?page=WPP_modify_website');

/** Constant: URL location for showing the list of groups in the portfolio. */
define('WPP_GROUP_SUMMARY', 					'admin.php?page=WPP_website_groups');

/** Constant: URL location for modifying a group entry. */
define('WPP_MODIFY_GROUP', 						'admin.php?page=WPP_modify_group');

/** Constant: The current version of the database needed by this version of the plugin.  */
define('WPP_VERSION', 							'1.0');


/**
 * Function: WPPortfolio_menu()
 *
 * Creates the menu with all of the configuration settings.
 */
add_action('admin_menu', 'WPPortfolio_menu');
function WPPortfolio_menu()
{
	add_menu_page('WP Portfolio - Summary of Websites in your Portfolio', 'WP Portfolio', 8, __FILE__, 'WPPortfolio_show_websites');
	
	add_submenu_page(__FILE__, 'WP Portfolio - Modify Website', 'Add New Website', 8, 'WPP_modify_website', 'WPPortfolio_modify_website');
	add_submenu_page(__FILE__, 'WP Portfolio - Modify Group', 'Add New Group', 8, 'WPP_modify_group', 'WPPortfolio_modify_group');
	add_submenu_page(__FILE__, 'WP Portfolio - Groups', 'Website Groups', 8, 'WPP_website_groups', 'WPPortfolio_website_groups');		
	add_submenu_page(__FILE__, 'WP Portfolio - General Settings', 'Portfolio Settings', 8, 'WPP_show_settings', 'WPP_show_settings_page');
	add_submenu_page(__FILE__, 'WP Portfolio - Layout Settings', 'Layout Settings', 8, 'WPP_show_layout_settings', 'WPP_show_layout_settings_page');
}


/**
 * Show the main settings page.
 */
function WPP_show_settings_page() {
?>
	<div class="wrap">
	<div id="icon-options-general" class="icon32">
	<br/>
	</div>
	<h2>WP Portfolio - General Settings</h2>
<?php 	

	// Get all the options from the database for the form
	$setting_stw_access_key = stripslashes(get_option('WPPortfolio_setting_stw_access_key'));
	$setting_stw_secret_key = stripslashes(get_option('WPPortfolio_setting_stw_secret_key'));
	$setting_stw_thumb_size = stripslashes(get_option('WPPortfolio_setting_stw_thumb_size'));
	$setting_cache_days 	= stripslashes(get_option('WPPortfolio_setting_cache_days'));
	$setting_fetch_method	= get_option('WPPortfolio_setting_fetch_method');
	$setting_show_credit	= get_option('WPPortfolio_setting_show_credit');
		
	// If we don't have the version in the settings, we're not installed
	if (!get_option('WPPortfolio_version')) {
		WPPortfolio_showMessage("No WP Portfolio settings were found, so it appears that the plugin has been uninstalled. Please <b>deactivate</b> and then <b>activate</b> the WP Portfolio plugin again to fix this.", true);
		return false;
	}
	
		
	// Uninstall plugin?
	if ($_GET['uninstall'] == "yes")
	{
		if ($_GET['confirm'] == "yes") {
			WPPortfolio_uninstall();
		}
		else {
			WPPortfolio_showMessage('Are you sure you want to delete all WP Portfolio settings and data? This action cannot be undone! </strong><br/><br/><a href="'.WPP_SETTINGS.'&uninstall=yes&confirm=yes">Yes, delete.</a> &nbsp; <a href="'.WPP_SETTINGS.'">NO!</a>');
		}
		return false;
	} // end if ($_GET['uninstall'] == "yes")	
	
	// Check if clearing cache 
	else if ( isset($_POST) && isset($_POST['clear_thumb_cache']) ) 
	{
		$actualThumbPath = trailingslashit(trailingslashit(WP_PLUGIN_DIR).WPP_THUMBNAIL_PATH);
		WPPortfolio_unlinkRecursive($actualThumbPath, false);
		
		WPPortfolio_showMessage("Thumbnail cache has now been emptied.");		
	}
	
	// Check if updated data.
	else if ( isset($_POST) && isset($_POST['update']) )
	{
		$setting_stw_access_key   = trim($_POST['setting_stw_access_key']);
		$setting_stw_secret_key   = trim($_POST['setting_stw_secret_key']);
		$setting_stw_thumb_size   = trim($_POST['setting_stw_thumb_size']);
		$setting_cache_days   	  = trim($_POST['setting_cache_days']);	
		$setting_fetch_method  	  = trim($_POST['setting_fetch_method']);
		$setting_show_credit	  = trim($_POST['setting_show_credit']);						
				
		// Validate keys
		if (WPPortfolio_isValidKey($setting_stw_access_key) && 
			WPPortfolio_isValidKey($setting_stw_secret_key))
		{		
			// Save settings
			update_option('WPPortfolio_setting_stw_access_key',   	$setting_stw_access_key);
			update_option('WPPortfolio_setting_stw_secret_key',   	$setting_stw_secret_key);
			update_option('WPPortfolio_setting_stw_thumb_size',   	$setting_stw_thumb_size);		
			update_option('WPPortfolio_setting_cache_days',   	  	$setting_cache_days);
			update_option('WPPortfolio_setting_fetch_method',     	$setting_fetch_method);
			update_option('WPPortfolio_setting_show_credit',     	$setting_show_credit);
								
			WPPortfolio_showMessage();		
		}
		else {
			WPPortfolio_showMessage("The keys must only contain letters and numbers. Please check that they are correct.", true);
		}
	}	
	
	
	$form = new FormBuilder();
	
	$formElem = new FormElement("setting_stw_access_key", "STW Access Key ID");
	$formElem->value = $setting_stw_access_key;
	$formElem->description = "The <a href=\"#doc-stw\">Shrink The Web</a> Access Key ID is around 15 characters.";
	$form->addFormElement($formElem);
	
	$formElem = new FormElement("setting_stw_secret_key", "STW Secret Key");
	$formElem->value = $setting_stw_secret_key;
	$formElem->description = "The <a href=\"#doc-stw\">Shrink The Web</a> Secret Key is around 5-10 characters. This key is never shared,<br>it is only stored in your settings and used to generate thumbnails for this website.";
	$form->addFormElement($formElem);
	
	// Thumbnail sizes
	$thumbsizes = array ("sm" => "Small (120 x 90)",
						 "lg" => "Large (200 x 150)",
						 "xlg" => "Extra Large (320 x 240)");
	
	$formElem = new FormElement("setting_stw_thumb_size", "Thumbnail Size");
	$formElem->value = $setting_stw_thumb_size;
	$formElem->setTypeAsComboBox($thumbsizes);
	$form->addFormElement($formElem);
	
	// Cache days
	$cachedays = array ( "3" => "3 days",
						 "5" => "5 days",
						 "7" => "7 days",
						 "10" => "10 days",
						 "15" => "15 days",
						 "20" => "20 days",
						 "30" => "30 days");
	
	$formElem = new FormElement("setting_cache_days", "Number of Days to Cache Thumbnail");
	$formElem->value = $setting_cache_days;
	$formElem->setTypeAsComboBox($cachedays);
	$form->addFormElement($formElem);	
	
	// Thumbnail Fetch Method
	$fetchmethod = array( "curl" => "cURL (recommended)",
						  "fopen" => "fopen");
	
	$formElem = new FormElement("setting_fetch_method", "Thumbnail Fetch Method");
	$formElem->value = $setting_fetch_method;
	$formElem->setTypeAsComboBox($fetchmethod);
	$formElem->description = "The type of HTTP call used to fetch thumbnails. fopen is usually less secure and disabled by most web hosts, hence why cURL is recommended.";
	$form->addFormElement($formElem);		
	
	// Show credit link
	$formElem = new FormElement("setting_show_credit", "Show Credit Link");
	$formElem->value = $setting_show_credit;
	$formElem->setTypeAsCheckbox("Creates a link back to WP Portfolio and to DanHarrison.co.uk.");
	$formElem->description = "<strong>I've worked hard on this plugin, please consider keeping the link back to my site!</strong> It's the link back to my site that keeps this plugin free!";
	$form->addFormElement($formElem);	
			
	$form->addButton("clear_thumb_cache", "Clear Thumbnail Cache");
	
	echo $form->toString();
	
	?>	

	
	<h2>Useful Information</h2>
	
	<h3>Comments and Feedback</h3>
	<p>Hopefully you find this plugin useful. If you have any comments, ideas or any other feedback on this plugin, please get in touch via my website <a href="http://www.danharrison.co.uk">DanHarrison.co.uk</a>.</p>	
	
	<a name="doc-stw"></a>
	<h3>Shrink The Web - Thumbnail Service</h3>
	<p>In order to use this plugin, you'll need to create an account on <a href="http://www.shrinktheweb.com" target="_blank">Shrink The Web (STW)</a>. This is a free service that gives you a number of free thumbnails per month. The WP Portfolio plugin employs the use of caching for efficiency and to reduce the number of times the STW service is accessed. You can check your thumbnail usage on the STW website at any time.</p>
	
	
	<h3>Uninstalling WP Portfolio</h3>
	<p>If you're going to permanently uninstall WP Portfolio, you can also <a href="admin.php?page=WPP_show_settings&uninstall=yes">remove all settings and data</a>.</p>
		
	<p>&nbsp;</p>	
	<p>&nbsp;</p>
</div>
<?php 
}

/**
 * Show only the settings relating to layout of the portfolio.
 */
function WPP_show_layout_settings_page() 
{
?>
	<div class="wrap">
	<div id="icon-themes" class="icon32">
	<br/>
	</div>
	<h2>WP Portfolio - Layout Settings</h2>
<?php 	

	// Get all the options from the database for the form
	$setting_template_website = stripslashes(get_option('WPPortfolio_setting_template_website'));
	$setting_template_group   = stripslashes(get_option('WPPortfolio_setting_template_group'));	
	$setting_template_css     = stripslashes(get_option('WPPortfolio_setting_template_css'));
	
	
	// If we don't have the version in the settings, we're not installed
	if (!get_option('WPPortfolio_version')) {
		WPPortfolio_showMessage("No WP Portfolio settings were found, so it appears that the plugin has been uninstalled. Please <b>deactivate</b> and then <b>activate</b> the WP Portfolio plugin again to fix this.", true);
		return false;
	}
	
			
	// Check if updated data.
	if ( isset($_POST) && isset($_POST['update']) )
	{
		$setting_template_website = trim($_POST['setting_template_website']);
		$setting_template_group   = trim($_POST['setting_template_group']);
		$setting_template_css     = trim($_POST['setting_template_css']);
		
		update_option('WPPortfolio_setting_template_website', $setting_template_website);
		update_option('WPPortfolio_setting_template_group',   $setting_template_group);			
		update_option('WPPortfolio_setting_template_css',     $setting_template_css);
				
		WPPortfolio_showMessage();		
	}	
	
	
	$form = new FormBuilder();	
	
	$formElem = new FormElement("setting_template_website", "Website HTML Template");				
	$formElem->value = htmlentities(stripslashes($setting_template_website));
	$formElem->description = "This is the template used to render each of the websites.";
	$formElem->setTypeAsTextArea(8, 70); 
	$form->addFormElement($formElem);
	// TODO class="form-field"
	
	$formElem = new FormElement("setting_template_group", "Group HTML Template");				
	$formElem->value = htmlentities(stripslashes($setting_template_group));
	$formElem->description = "This is the template used to render each of the groups that the websites belong to.";
	$formElem->setTypeAsTextArea(3, 70); 
	$form->addFormElement($formElem);	
	
	$formElem = new FormElement("setting_template_css", "Template CSS");				
	$formElem->value = htmlentities(stripslashes($setting_template_css));
	$formElem->description = "This is the CSS code used to style the portfolio. <strong>Advanced Tip:</strong> Once you're happy with the style, you should really move the CSS into your template <tt>style.css</tt>. This is so that visitor browsers can cache the stylesheet and reduce loading times. Any CSS placed here will be injected into the template &lt;head&gt; tag, which is not the most efficient method of delivering CSS.";
	$formElem->setTypeAsTextArea(10, 70); 
	$form->addFormElement($formElem);	
	
	
	echo $form->toString();
	
	?>	

	
	<h2>Default Templates</h2>	
	
	<p>The default templates for the groups and websites below as a reference.</p>
	<ul style="margin-left: 30px; list-style-type: disc;">
		<li><strong><?php echo WPP_STR_GROUP_NAME; ?></strong> - Replace with the group name.</li>
		<li><strong><?php echo WPP_STR_GROUP_DESCRIPTION; ?></strong> - Replace with the group description.</li>
		<li><strong><?php echo WPP_STR_WEBSITE_NAME; ?></strong> - Replace with the website name.</li>
		<li><strong><?php echo WPP_STR_WEBSITE_URL; ?></strong> - Replace with the website url.</li>
		<li><strong><?php echo WPP_STR_WEBSITE_DESCRIPTION; ?></strong> - Replace with the website description.</li>
		<li><strong><?php echo WPP_STR_WEBSITE_THUMBNAIL; ?></strong> - Replace with the website thumbnail including the &lt;img&gt; tag.</li>
	</ul>
	
	<form>
	<table class="form-table">
		<tr class="form-field">
			<th scope="row"><label for="default_template_group">Group Template</label></th>
			<td>
				<textarea name="default_template_group" rows="3"><?php echo htmlentities(WPP_DEFAULT_GROUP_TEMPLATE); ?></textarea>
			</td>
		</tr>		
		<tr class="form-field">
			<th scope="row"><label for="default_template_website">Website Template</label></th>
			<td>
				<textarea name="default_template_website" rows="8"><?php echo htmlentities(WPP_DEFAULT_WEBSITE_TEMPLATE); ?></textarea>
			</td>
		</tr>			
		<tr class="form-field">
			<th scope="row"><label for="default_template_css">Website Template</label></th>
			<td>
				<textarea name="default_template_css" rows="8"><?php echo htmlentities(WPP_DEFAULT_CSS); ?></textarea>
			</td>
		</tr>					
	</table>
	</form>
	
	<p>&nbsp;</p>	
	<p>&nbsp;</p>
</div>
<?php 
}

function WPPortfolio_show_websites()
{
?>
<div class="wrap">
	<div id="icon-themes" class="icon32">
	<br/>
	</div>
	<h2>Summary of Websites in your Portfolio</h2>
	<br>
<?php 		
	
	global $wpdb;
	$websites_table = $wpdb->prefix . TABLE_WEBSITES;
	$groups_table   = $wpdb->prefix . TABLE_WEBSITE_GROUPS;
	
	// Get website details, merge with group details
	$SQL = "SELECT * FROM $websites_table
			LEFT JOIN $groups_table ON $websites_table.sitegroup = $groups_table.groupid
	 		ORDER BY $websites_table.sitename";	
		
	// DEBUG - Uncomment if needed
	//$wpdb->show_errors();
	$websites = $wpdb->get_results($SQL, OBJECT);	
	
	// Only show table if there are websites to show
	if ($websites)
	{
		?>
		<table class="widefat post fixed">
		<thead>
			<tr>
				<th scope="col" width="40">ID</th>
				<th scope="col" width="130">Thumbnail</th>
				<th scope="col" width="200">Site Name</th>
				<th scope="col" width="100%">URL</th>			
				<th scope="col" width=150">Group</th>
				<th class="action-links" scope="col" width="100">Action</th>
			</tr>
		</thead>
		<tfoot>
			<tr>
				<th scope="col" width="40">ID</th>
				<th scope="col" width="130">Thumbnail</th>
				<th scope="col" width="200">Site Name</th>
				<th scope="col" width="100%">URL</th>			
				<th scope="col" width=150">Group</th>
				<th class="action-links" scope="col" width="100">Action</th>
			</tr>
		</tfoot>
		<tbody>
		<?php
			
		foreach ($websites as $websitedetails)
		{
			echo '<tr class="alternate">';
			echo '<td>'.$websitedetails->siteid.'</td>';			

			// Hyperlink the thumbnail
			echo '<td><a href="'.$websitedetails->siteurl.'" target="_new"><img src="'.getThumbnail($websitedetails->siteurl, "sm").'" width="120" height="90" style="border: 1px solid #CCC; padding: 2px;" /></a></td>';
						
			echo '<td>'.stripslashes($websitedetails->sitename).'</td>';
			
			// Create a clickable URL
			echo '<td><a href="'.$websitedetails->siteurl.'" target="_new">'.$websitedetails->siteurl.'</a></td>';
						
			echo '<td>'.stripslashes($websitedetails->groupname).'</td>';
			echo '<td class="action-links"><a href="'.WPP_MODIFY_WEBSITE.'&delete=yes&siteid='.$websitedetails->siteid.'">Delete</a>&nbsp;|&nbsp;';
			echo '<a href="'.WPP_MODIFY_WEBSITE.'&editmode=yes&siteid='.$websitedetails->siteid.'">Edit</a></td>';
			echo '</tr>';
			
			// If fetching thumbnails, this might take a while. So flush.
			flush();
		}
	} // end of if websites
	else {
		WPPortfolio_showMessage("There are currently no websites in the portfolio.", true);
	}
	?>
	</tbody>
</table>	
</div>
<?php 
	
}

/**
 * Shows the page that allows the details of a website to be modified or added to the portfolio.
 */
function WPPortfolio_modify_website()
{
	// Determine if we're in edit mode
	$editmode = isset($_GET['editmode']);
	$delete   = isset($_GET['delete']);

	$verb = "Add New";
	if ($delete)
	$verb = "Delete";
	else if ($editmode)
	$verb = "Modify"
	
	// Show title to determine action
	?>
	<div class="wrap">
	<div id="icon-themes" class="icon32">
	<br/>
	</div>
	<h2><?php echo $verb; ?> Website Details</h2>
	<?php 
	
	// Check id is a valid number if editing $editmode
	if ($editmode && !is_numeric($_GET['siteid'])) {
		WPPortfolio_showMessage('Sorry, but no website with that ID could be found. Please click <a href="'.WPP_WEBSITE_SUMMARY.'">here</a> to return to the list of websites.', true);
		return;
	}	
	
	// Check if we're deleting a website
	if (isset($_GET['siteid']) && isset($_GET['delete']))
	{
		$siteid = $_GET['siteid'];
		
		global $wpdb;
		$table_name      = $wpdb->prefix . TABLE_WEBSITES;
		$websitedetails  = $wpdb->get_row("SELECT * FROM $table_name WHERE siteid = '".$wpdb->escape($siteid)."' LIMIT 1", OBJECT);
		
		if (isset($_GET['confirm']))
		{
			$delete_website = "DELETE FROM $table_name WHERE siteid = '".$wpdb->escape($siteid)."' LIMIT 1";
			if ($wpdb->query( $delete_website )) {
				WPPortfolio_showMessage("Website was successfully deleted.");
			}
			else {
				WPPortfolio_showMessage("Sorry, but an unknown error occured whist trying to delete the selected website from the portfolio.", true);
			}
		}
		else
		{
			$message = 'Are you sure you want to delete "'.$websitedetails->sitename.'" from your portfolio?<br/><br/><a href="'.WPP_MODIFY_WEBSITE.'&delete=yes&confirm=yes&siteid='.$websitedetails->siteid.'">Yes, delete.</a> &nbsp; <a href="'.WPP_WEBSITE_SUMMARY.'">NO!</a>';
			WPPortfolio_showMessage($message);
			return;
		}
	}	

	// Check if we're adding or editing a website
	if ($editmode && isset($_GET['siteid']))
	{
		// Get details from the database
		global $wpdb;
		$table_name = $wpdb->prefix . TABLE_WEBSITES;
		
		$SQL = "SELECT * FROM $table_name 
				WHERE siteid = '".$wpdb->escape($_GET['siteid'])."' LIMIT 1";
		
		$websitedetails = $wpdb->get_row($SQL, OBJECT);

		// Found an website with matching ID
		if ($websitedetails)
		{
			// Grab details
			$website_siteid 		 = $websitedetails->siteid;
			$website_sitename 		 = stripslashes($websitedetails->sitename);
			$website_siteurl 		 = stripslashes($websitedetails->siteurl);
			$website_sitedescription = stripslashes($websitedetails->sitedescription);
			$website_sitegroup 		 = $websitedetails->sitegroup;
		}
		// False alarm, couldn't find it.
		else {
			$editmode = false;
		}
		
		
	} // end of editing check
	
	
	// Check if website is being added, if so, add to the database.
	if ( isset($_POST) && isset($_POST['update']) )
	{
		// Grab specified details
		$website_siteid 		 = $_POST['website_siteid'];
		$website_sitename 		 = strip_tags($_POST['website_sitename']);
		$website_siteurl 		 = strip_tags($_POST['website_siteurl']);
		$website_sitedescription = $_POST['website_sitedescription'];
		$website_sitegroup 		 = $_POST['website_sitegroup'];
		
		// Keep track of errors for validation
		$errors = array();
				
		// Ensure all fields have been completed
		if (!($website_sitename && $website_siteurl && $website_sitedescription)) {
			array_push($errors, "Please check that you have completed the site name, url and description fields.");
		}	
		
		// Continue if there are no errors
		if (count($errors) == 0)
		{
			global $wpdb;
			$table_name = $wpdb->prefix . TABLE_WEBSITES;

			// Change query based on add or edit
			if ($_POST['update'] == "add")
			{				
				$query = "INSERT INTO " . $table_name . " (sitename, siteurl, sitedescription, sitegroup) " .
						 "VALUES ('" . 	$wpdb->escape($website_sitename) . "', '" . 
										$wpdb->escape($website_siteurl) . "', '" .
										$wpdb->escape($website_sitedescription) . "', '" . 
										$wpdb->escape($website_sitegroup) .  
										"')";
			}

			// Edit
			else
			{
				$query = "UPDATE " . $table_name . " SET " .
						 "sitename = '" 			. $wpdb->escape($website_sitename) . 
						 "', siteurl = '" 			. $wpdb->escape($website_siteurl) . 
						 "', sitedescription = '" 	. $wpdb->escape($website_sitedescription) .
						 "', sitegroup = '" 		. $wpdb->escape($website_sitegroup) . 
						 "' " .
						 "WHERE siteid = '" 		. $wpdb->escape($website_siteid). "'";
			}
			
			// Try to put the data into the database
			$wpdb->show_errors();
			$wpdb->query($query);

			
			// When adding, clean fields so that we don't show them again.
			if ($editmode) {
				WPPortfolio_showMessage("Website details successfully updated.");
			} 
			else
			{								
				$website_siteid 		 = "";
				$website_sitename 		 = "";
				$website_siteurl 		 = "";
				$website_sitedescription = "";
				$website_sitegroup 		 = "";
				
				WPPortfolio_showMessage("Website details successfully added.");
			}
								
		} // end of error checking
	
		// Handle error messages
		else
		{
			$message = "Sorry, but unfortunately there were some errors. Please fix the errors and try again.<br><br>";
			$message .= "<ul style=\"margin-left: 20px; list-style-type: square;\">";
			
			// Loop through all errors in the $error list
			foreach ($errors as $errormsg) {
				$message .= "<li>$errormsg</li>";
			}
						
			$message .= "</ul>";
			WPPortfolio_showMessage($message, true);
		}
	}
	
	
	?>
	<form method="post" action="<?php echo str_replace( '%7E', '~', $_SERVER['REQUEST_URI']); ?>">
	
	<table class="form-table">
		<tr class="form-field form-required">
			<th scope="row" valign="top"><label for="website_sitename">Website Name</label></th>
			<td>
				<input type="text" name="website_sitename"	value="<?php echo $website_sitename; ?>" maxlength="50" class="regular-text" />
				<span class="setting-description"><br>The proper name of the website.</span>
			</td>
		</tr>
		
		<tr class="form-field form-required">
			<th scope="row" valign="top"><label for="website_siteurl">Website URL</label></th>
			<td>
				<input type="text" name="website_siteurl" value="<?php echo $website_siteurl; ?>" maxlength="255" class="url" />
				<span class="setting-description"><br>The URL for the website, including the leading <em>http://</em>.</span>
			</td>
		</tr>		
		
		<tr class="form-field form-required">
			<th scope="row" valign="top"><label for="website_sitedescription">Website Description</label></th>
			<td>
				<textarea name="website_sitedescription" rows="4"><?php echo $website_sitedescription; ?></textarea>
				<span class="setting-description"><br>The description of your website. HTML is permitted.</span>
			</td>
		</tr>
		
		<tr class="form-field form-required">
			<th scope="row" valign="top"><label for="website_sitegroup">Website Group</label></th>
			<td>
				<?php echo WPPortfolio_getListOfWebsiteGroups($website_sitegroup, "website_sitegroup"); ?>
				<span class="setting-description"><br>The group you want to assign this website to.</span>
			</td>
		</tr>	

	</table>
	
	<p class="submit">
		<input type="submit" name="Submit" class="button-primary" value="<?php echo ($editmode ? "Update" : "Add"); ?> Website Details" />
		<input type="hidden" name="update" value="<?php echo ($editmode ? "edit" : "add"); ?>" />
		<input type="hidden" name="website_siteid" value="<?php echo $website_siteid; ?>" /> 
		<input type="hidden" name="page_options" value="website_siteid, website_sitename, website_sitedescription, website_sitegroup, website_siteurl" />
	</p>
	</form>
	
		
	<br><br>
	</div><!-- wrap -->
	<?php 	
}


/**
 * Shows the page that allows a group to be modified.
 */
function WPPortfolio_modify_group()
{
	// Determine if we're in edit mode
	$editmode = isset($_GET['editmode']);
	$delete   = isset($_GET['delete']);

	$verb = "Add New";
	if ($delete)
	$verb = "Delete";
	else if ($editmode)
	$verb = "Modify"
	
	// Show title to determine action
	?>
	<div class="wrap">
	<div id="icon-edit" class="icon32">
	<br/>
	</div>
	<h2><?php echo $verb; ?> Group Details</h2>
	<?php 
	
	// Check id is a valid number if editing $editmode
	if ($editmode && !is_numeric($_GET['groupid'])) {
		WPPortfolio_showMessage('Sorry, but no group with that ID could be found. Please click <a href="'.WPP_GROUP_SUMMARY.'">here</a> to return to the list of groups.', true);
		return;
	}	

		
	// Check if we're deleting a group
	if (isset($_GET['groupid']) && isset($_GET['delete']))
	{
		// Group ID should be a real number at this point.
		$groupid = $_GET['groupid'];
						
		global $wpdb;
		$groups_table = $wpdb->prefix . TABLE_WEBSITE_GROUPS;
		$websites_table = $wpdb->prefix . TABLE_WEBSITES;
		
		// Now check that ID actually relates to a real group
		$groupdetails = $wpdb->get_row("SELECT * FROM $groups_table WHERE groupid = '".$wpdb->escape($groupid)."' LIMIT 1", OBJECT);
		
		// If group doesn't really exist, then stop.
		if (!$groupdetails) {
			WPPortfolio_showMessage('Sorry, but no group with that ID could be found. Please click <a href="'.WPP_GROUP_SUMMARY.'">here</a> to return to the list of groups.', true);
			return;
		}
		
		// Count the number of websites in this group and how many groups exist
		$website_count = $wpdb->get_var("SELECT COUNT(*) FROM $websites_table WHERE sitegroup = '".$wpdb->escape($groupdetails->groupid)."'");
		$group_count   = $wpdb->get_var("SELECT COUNT(*) FROM $groups_table");
		
		// Check that group doesn't have a load of websites assigned to it.
		if ($website_count > 0)  {
			WPPortfolio_showMessage("Sorry, the group '$groupdetails->groupname' still contains <b>$website_count</b> websites. Please ensure the group is empty before deleting it.");
			return;
		}
		
		// If we're deleting the last group, don't let it happen
		if ($group_count == 1)  {
			WPPortfolio_showMessage("Sorry, but there needs to be at least 1 group in the portfolio. Please add a new group before deleting '$groupdetails->groupname'.");
			return;
		}
		
		// OK, got this far, confirm we want to delete.
		if (isset($_GET['confirm']))
		{
			$delete_group = "DELETE FROM $groups_table WHERE groupid = '".$wpdb->escape($groupid)."' LIMIT 1";
			if ($wpdb->query( $delete_group )) {
				WPPortfolio_showMessage("Group was successfully deleted.");
			}
			else {
				WPPortfolio_showMessage("Sorry, but an unknown error occured whist trying to delete the selected group from the portfolio.", true);
			}
		}
		else
		{
			$message = 'Are you sure you want to delete the group "'.$groupdetails->groupname.'" from your portfolio?<br/><br/><a href="'.WPP_MODIFY_GROUP.'&delete=yes&confirm=yes&groupid='.$groupdetails->groupid.'">Yes, delete.</a> &nbsp; <a href="'.WPP_GROUP_SUMMARY.'">NO!</a>';
			WPPortfolio_showMessage($message);
			return;
		}
	}	

	// Check if we're adding or editing a group
	if ($editmode && isset($_GET['groupid']))
	{
		// Get details from the database
		global $wpdb;
		$table_name = $wpdb->prefix . TABLE_WEBSITE_GROUPS;
		
		$SQL = "SELECT * FROM $table_name 
				WHERE groupid = '".$wpdb->escape($_GET['groupid'])."' LIMIT 1";
		
		$groupdetails = $wpdb->get_row($SQL, OBJECT);

		// Found an group with matching ID
		if ($groupdetails)
		{
			// Grab details
			$group_groupid  		= $groupdetails->groupid;
			$group_groupname        = stripslashes($groupdetails->groupname);
			$group_groupdescription = stripslashes($groupdetails->groupdescription);
			$group_grouporder       = $groupdetails->grouporder;
		}
		// False alarm, couldn't find it.
		else {
			$editmode = false;
		}
		
		
	} // end of editing check
	
	
	// Check if group is being added, if so, add to the database.
	if ( isset($_POST) && isset($_POST['update']) )
	{
		// Grab specified details
		$group_groupid  		= $_POST['group_groupid'];
		$group_groupname        = strip_tags($_POST['group_groupname']);
		$group_groupdescription = $_POST['group_groupdescription'];
		$group_grouporder       = $_POST['group_grouporder'] + 0; // Add zero to convert to number
				
		// Keep track of errors for validation
		$errors = array();
				
		// Ensure all fields have been completed
		if (!($group_groupname && $group_groupdescription)) {
			array_push($errors, "Please check that you have completed the group name and description fields.");
		}	
		
		// Continue if there are no errors
		if (count($errors) == 0)
		{
			global $wpdb;
			$table_name = $wpdb->prefix . TABLE_WEBSITE_GROUPS;

			// Change query based on add or edit
			if ($_POST['update'] == "add")
			{				
				$query = "INSERT INTO " . $table_name . " (groupname, groupdescription, grouporder) " .
						 "VALUES ('" . 	$wpdb->escape($group_groupname) . "', '" . 
										$wpdb->escape($group_groupdescription) . "', '" . 
										$wpdb->escape($group_grouporder) .  
										"')";
			}

			// Edit
			else
			{
				$query = "UPDATE " . $table_name . " SET " .
						 "groupname = '" 			. $wpdb->escape($group_groupname) .  
						 "', groupdescription = '" 	. $wpdb->escape($group_groupdescription) .
						 "', grouporder = '" 		. $wpdb->escape($group_grouporder) . 
						 "' " .
						 "WHERE groupid = '" 		. $wpdb->escape($group_groupid). "'";
			}
			
			// Try to put the data into the database
			$wpdb->show_errors();
			$wpdb->query($query);
			
			// When adding, clean fields so that we don't show them again.
			if ($editmode) {
				WPPortfolio_showMessage("Group details successfully updated.");
			} 
			else
			{								
				$group_groupid  		= false;
				$group_groupname        = "";
				$group_groupdescription = "";
				$group_grouporder       = false;				
				
				WPPortfolio_showMessage("Group details successfully added.");
			}
								
		} // end of error checking
	
		// Handle error messages
		else
		{
			$message = "Sorry, but unfortunately there were some errors. Please fix the errors and try again.<br><br>";
			$message .= "<ul style=\"margin-left: 20px; list-style-type: square;\">";
			
			// Loop through all errors in the $error list
			foreach ($errors as $errormsg) {
				$message .= "<li>$errormsg</li>";
			}
						
			$message .= "</ul>";
			WPPortfolio_showMessage($message, true);
		}
	}
	
	
	?>
	<form method="post" action="<?php echo str_replace( '%7E', '~', $_SERVER['REQUEST_URI']); ?>">
	
	<table class="form-table">
		<tr class="form-field form-required">
			<th scope="row" valign="top"><label for="group_groupname">Group Name</label></th>
			<td>
				<input type="text" name="group_groupname" value="<?php echo $group_groupname; ?>" maxlength="100" class="regular-text" />
				<span class="setting-description"><br>The name for this group of websites.</span>
			</td>
		</tr>
				
		<tr class="form-field form-required">
			<th scope="row" valign="top"><label for="group_groupdescription">Group Description</label></th>
			<td>
				<textarea name="group_groupdescription" rows="4"><?php echo $group_groupdescription; ?></textarea>
				<span class="setting-description"><br>The description of your group. HTML is permitted.</span>
			</td>
		</tr>
		
		<tr class="form-field form-required">
			<th scope="row" valign="top"><label for="group_grouporder">Group Order</label></th>
			<td>
				<input type="text" name="group_grouporder" value="<?php echo $group_grouporder; ?>" maxlength="100" class="regular-text" />
				<span class="setting-description"><br>The number to use for ordering the groups. Groups are rendered in ascending order, first by this order value (lowest value first), then by group name. 
				e.g. Groups (A, B, C, D) with ordering (50, 100, 0, 50) will be rendered as (C, A, D, B).  
				If all groups have 0 for ordering, then the groups are rendered in alphabetical order. </span>
			</td>
		</tr>
	
	</table>
	
	<p class="submit">
		<input type="submit" name="Submit" class="button-primary" value="<?php echo ($editmode ? "Update" : "Add"); ?> Group Details" />
		<input type="hidden" name="update" value="<?php echo ($editmode ? "edit" : "add"); ?>" />
		<input type="hidden" name="group_groupid" value="<?php echo $group_groupid; ?>" /> 
		<input type="hidden" name="page_options" value="group_groupid, group_groupname, group_groupdescription, website_siteurl" />
	</p>
	</form>
	
		
	<br><br>
	</div><!-- wrap -->
	<?php 	

	
}

/**
 * Shows the page listing the available groups.
 */
function WPPortfolio_website_groups()
{
?>
<div class="wrap">
	<div id="icon-edit" class="icon32">
	<br/>
	</div>
	<h2>Website Groups</h2>
	<br/>

	<?php 
	global $wpdb;
	$groups_table = $wpdb->prefix . TABLE_WEBSITE_GROUPS;
	$websites_table = $wpdb->prefix . TABLE_WEBSITES;
	
	// Get website details, merge with group details
	$SQL = "SELECT * FROM $groups_table
	 		ORDER BY grouporder, groupname";	
	
	// DEBUG Uncomment if needed
	// $wpdb->show_errors();
	$groups = $wpdb->get_results($SQL, OBJECT);
	
	// Only show table if there are any results.
	if ($groups)
	{		
		?>
		<p>The websites will be rendered in groups in the order shown in the table.</p>
		<table class="widefat post fixed">
		<thead>
			<tr>
				<th scope="col" width="40">ID</th>
				<th scope="col" width="200">Name</th>			
				<th scope="col" width="100%">Description</th>
				<th scope="col" width="150" style="text-align: center;"># Websites</th>
				<th scope="col" width="100" style="text-align: center;">Ordering</th>
				<th class="action-links" scope="col" width="100">Action</th>
			</tr>
		</thead>
		<tfoot>
			<tr>
				<th scope="col" width="40">ID</th>
				<th scope="col" width="200">Name</th>			
				<th scope="col" width="100%">Description</th>
				<th scope="col" width="150" style="text-align: center;"># Websites</th>
				<th scope="col" width="100" style="text-align: center;">Ordering</th>
				<th class="action-links" scope="col" width="100">Action</th>
			</tr>
		</tfoot>
		<tbody>
		<?php		
		
		foreach ($groups as $groupdetails)
		{
			echo '<tr class="alternate">';
			echo '<td>'.$groupdetails->groupid.'</td>';									
			echo '<td>'.stripslashes($groupdetails->groupname).'</td>';
			echo '<td>'.stripslashes($groupdetails->groupdescription).'</td>';
			
			// Count websites in this group
			$website_count = $wpdb->get_var("SELECT COUNT(*) FROM $websites_table WHERE sitegroup = '".$wpdb->escape($groupdetails->groupid)."'");
			
			echo '<td style="text-align: center;">'.$website_count.($website_count == 1 ? ' website' : ' websites').'</td>';
			echo '<td style="text-align: center;">'.$groupdetails->grouporder.'</td>';
			
			echo '<td class="action-links"><a href="'.WPP_MODIFY_GROUP.'&delete=yes&groupid='.$groupdetails->groupid.'">Delete</a>&nbsp;|&nbsp;';
			echo '<a href="'.WPP_MODIFY_GROUP.'&editmode=yes&groupid='.$groupdetails->groupid.'">Edit</a></td>';
			echo '</tr>';
		}
	} // end of if groups
	// No groups to show
	else {
		WPPortfolio_showMessage("There are currently no groups in the portfolio.", true);
	}
	?>
	</tbody>
</table>	
</div>
<?php 
	
}


/**
 * Function: WPPortfolio_()
 * Install the WP Portfolio plugin, initialise the default settings, and create the tables for the websites and groups.
 */
function WPPortfolio_install()
{
	global $wpdb;

	// ### Create Default Settings
	if (!get_option('WPPortfolio_setting_stw_access_key'))
		update_option('WPPortfolio_setting_stw_access_key', "");
	
	if (!get_option('WPPortfolio_setting_stw_secret_key'))
		update_option('WPPortfolio_setting_stw_secret_key', "");
	
	if (!get_option('WPPortfolio_setting_stw_thumb_size'))
		update_option('WPPortfolio_setting_stw_thumb_size', "lg");
				
	if (!get_option('WPPortfolio_setting_cache_days'))
		update_option('WPPortfolio_setting_cache_days', 7);

	if (!get_option('WPPortfolio_setting_template_website'))
		update_option('WPPortfolio_setting_template_website', WPP_DEFAULT_WEBSITE_TEMPLATE);		
		
	if (!get_option('WPPortfolio_setting_template_group'))
		update_option('WPPortfolio_setting_template_group', WPP_DEFAULT_GROUP_TEMPLATE);

	if (!get_option('WPPortfolio_setting_template_css'))
		update_option('WPPortfolio_setting_template_css', WPP_DEFAULT_CSS);	
	
	if (!get_option('WPPortfolio_setting_fetch_method'))
		update_option('WPPortfolio_setting_fetch_method', "curl");	
		
		if (!get_option('WPPortfolio_setting_show_credit'))
		update_option('WPPortfolio_setting_show_credit', "on");
			
			
	$wpdb->show_errors();

	// #### Create Tables
	// Create TABLE_WEBSITES table if it doesn't exist.
	$table_name = $wpdb->prefix . TABLE_WEBSITES;	
	if($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) 
	{
		$sql = "CREATE TABLE `$table_name` (
  				   `siteid` int(10) unsigned NOT NULL auto_increment,
				   `sitename` varchar(150),
				   `siteurl` varchar(255),
				   `sitedescription` TEXT,
				   `sitegroup` int(10) unsigned NOT NULL,
				   PRIMARY KEY (`siteid`)
				) ";

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);
	}
	
	// Create TABLE_WEBSITE_GROUPS table if it doesn't exist.
	$table_name = $wpdb->prefix . TABLE_WEBSITE_GROUPS;	
	
	if($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) 
	{
		$sql = "CREATE TABLE `$table_name` (
  				   `groupid` int(10) UNSIGNED NOT NULL auto_increment,
				   `groupname` varchar(150),
				   `groupdescription` TEXT,
				    `grouporder` INT(1) UNSIGNED NOT NULL DEFAULT '0',
				   PRIMARY KEY (`groupid`)
				) ";
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);

		// Add default group that has ID of 0
		$SQL = "INSERT INTO `$table_name` (groupid, groupname, groupdescription) VALUES (1, 'My Websites', 'These are all my websites.')";
 		$results = $wpdb->query($SQL);
		
	}	

	
	// Update the version regardless
	update_option("WPPortfolio_version", WPP_VERSION);
}
register_activation_hook(__FILE__,'WPPortfolio_install');

/**
 * Recursively delete the specified directory name, including all its files.
 */
function rmdirr($dirname)
{
	// Sanity check
	if (!file_exists($dirname)) {
		return false;
	}

	// Simple delete for a file
	if (is_file($dirname) || is_link($dirname)) {
		return unlink($dirname);
	}
	
	// Loop through the folder
	$dir = dir($dirname);
	while (false !== $entry = $dir->read()) {
		// Skip pointers
		if ($entry == '.' || $entry == '..') {
			continue;
		}

		// Recurse
		rmdirr($dirname . DIRECTORY_SEPARATOR . $entry);
	}
 
	// Clean up
	$dir->close();
	return rmdir($dirname);
}

/**
 * Method called when we want to uninstall the portfolio plugin to remove the database tables.
 */
function WPPortfolio_uninstall() 
{
	// Remove all options from the database
	delete_option('WPPortfolio_setting_stw_access_key');
	delete_option('WPPortfolio_setting_stw_secret_key');	
	delete_option('WPPortfolio_setting_stw_thumb_size');
	delete_option('WPPortfolio_setting_cache_days');
	
	delete_option('WPPortfolio_setting_template_website');
	delete_option('WPPortfolio_setting_template_group');
	delete_option('WPPortfolio_setting_template_css');
			
	delete_option('WPPortfolio_version');
		
	
	// Remove all tables for the portfolio
	global $wpdb;
	$table_name    = $wpdb->prefix . TABLE_WEBSITES;
	$uninstall_sql = "DROP TABLE IF EXISTS ".$table_name;
	$wpdb->query($uninstall_sql);
	
	$table_name    = $wpdb->prefix . TABLE_WEBSITE_GROUPS;
	$uninstall_sql = "DROP TABLE IF EXISTS ".$table_name;
	$wpdb->query($uninstall_sql);
	
	// Remove cache
	$actualThumbPath = trailingslashit(trailingslashit(WP_PLUGIN_DIR).WPP_THUMBNAIL_PATH);
	rmdirr($actualThumbPath);		
	
	
	WPPortfolio_showMessage("Deleted WP Portfolio database entries.");
}




/**
 * This method is called just before the <head> tag is closed. We inject our custom CSS into the 
 * webpage here.
 */
function WPPortfolio_renderCSS() 
{
	// Only render CSS if there's some to show.
	$setting_template_css = trim(stripslashes(get_option('WPPortfolio_setting_template_css')));
	if ($setting_template_css) {
		echo "\n<!-- WP Portfolio Stylesheet -->\n";
		echo "<style type=\"text/css\">\n$setting_template_css\n</style>";
		echo "\n<!-- WP Portfolio Stylesheet -->\n";
	}
}
add_action('wp_head', 'WPPortfolio_renderCSS');



/**
 * Turn the portfolio of websites in the database into a single page containing details and screenshots.
 * @param $content The content to update.
 * @return String The updated content for the post or page.
 */
function WPPortfolio_renderPortfolio($oldcontent)
{	
	// Detect if we need to render the portfolio by looking for the 
	// special string <!-- ShowMyPortfolio -->
	$newcontent = $oldcontent;
	$content = "";	
	if (preg_match(WPP_STR_SHOW_PORTFOLIO, $oldcontent))
	{    	    	
		// Get portfolio from database
		global $wpdb;
		$websites_table = $wpdb->prefix . TABLE_WEBSITES;
		$groups_table   = $wpdb->prefix . TABLE_WEBSITE_GROUPS;
			
		// Get website details, merge with group details
		$SQL = "SELECT * FROM $websites_table
				LEFT JOIN $groups_table ON $websites_table.sitegroup = $groups_table.groupid
			 	ORDER BY grouporder, groupname, sitename";			
		
		$wpdb->show_errors();
		$websites = $wpdb->get_results($SQL, OBJECT);
	
		if ($websites)
		{	
			// Just put some space after other content before rendering portfolio.	
			$content .= "\n\n";			

			// Used to track what group we're working with.
			$prev_group = "";
			
			// Get templates to use for rendering the website details.
			$setting_template_website = stripslashes(get_option('WPPortfolio_setting_template_website'));
			$setting_template_group   = stripslashes(get_option('WPPortfolio_setting_template_group'));						
			
			// Render all the websites, but look after different groups
			foreach ($websites as $websitedetails)
			{
				// If we're rendering a new group, then show the group name and description 
				if ($prev_group != $websitedetails->groupname)
				{
					// Replace group name and description.					
					$renderedstr = WPPortfolio_replaceString(WPP_STR_GROUP_NAME, stripslashes($websitedetails->groupname), $setting_template_group);
					$renderedstr = WPPortfolio_replaceString(WPP_STR_GROUP_DESCRIPTION, stripslashes($websitedetails->groupdescription), $renderedstr);
					
					// Update content with templated group details
					$content .= "\n\n$renderedstr\n";
				}
				
				// Get the image thumbnail and generate the image tag. 
				$imagetag = '<img src="'.getThumbnail($websitedetails->siteurl).'"/>';
				
				// Render the website details
				$renderedstr = WPPortfolio_replaceString(WPP_STR_WEBSITE_NAME, 		 stripslashes($websitedetails->sitename), $setting_template_website);
				$renderedstr = WPPortfolio_replaceString(WPP_STR_WEBSITE_URL, 		 stripslashes($websitedetails->siteurl), $renderedstr);
				$renderedstr = WPPortfolio_replaceString(WPP_STR_WEBSITE_DESCRIPTION, stripslashes($websitedetails->sitedescription), $renderedstr);
				$renderedstr = WPPortfolio_replaceString(WPP_STR_WEBSITE_THUMBNAIL, 	 $imagetag, $renderedstr);
				
				$content .= "\n$renderedstr\n";
				
				// If fetching thumbnails, this might take a while. So flush.
				flush();
				
				// Track the groups
				$prev_group = $websitedetails->groupname;
			}
			
			// Credit link on portfolio. 
			if (get_option('WPPortfolio_setting_show_credit') == "on") {				
				$content .= "<div style=\"font-size: 8pt; font-family: Verdana; float: right;\">Created using <a href=\"http://wordpress.org/extend/plugins/wp-portfolio\">WP Portfolio</a> by <a href=\"http://www.danharrison.co.uk/\">Dan Harrison</a></div>";
			}
						
			// Add some space after the portfolio HTML 
			$content .= "\n\n";
			
			// Now replace search string with formatted portfolio
			$newcontent = WPPortfolio_replaceString(WPP_STR_SHOW_PORTFOLIO, $content, $oldcontent);			
		}    	
	}
	return $newcontent;
}
add_filter('the_content', 'WPPortfolio_renderPortfolio');


/**
 * Gets the thumbnail for the specified website, stores it in the cache, and then returns the 
 * relative path to the cached image.
 * 
 * @param $url The URL to get the thumbnail for.
 * @param $size_override If specified, use this size rather than the size in the settings.
 * @return String The full URL for the thumbnail stored in the cache.
 */
function getThumbnail($url, $size_override = false)
{
	// Cache directory
	$actualThumbPath = trailingslashit(trailingslashit(WP_PLUGIN_DIR).WPP_THUMBNAIL_PATH);
			
	// Create cache directory if it doesn't exist
	if (!file_exists($actualThumbPath)) {
		mkdir($actualThumbPath, 0777, true);		
	}
	
	// Get all the options from the database for the thumbnail    
    $args["STWAccessKeyId"] = stripslashes(get_option('WPPortfolio_setting_stw_access_key'));
    $args["stwu"] 			= stripslashes(get_option('WPPortfolio_setting_stw_secret_key'));
    $setting_cache_days		= stripslashes(get_option('WPPortfolio_setting_cache_days'));
    $setting_fetch_method   = stripslashes(get_option('WPPortfolio_setting_fetch_method'));
        
    // If $size_override is specified, then use that size rather than the size stored in the settings.
    if ($size_override) {
    	$args["Size"] = $size_override;
    } else { 
    	$args["Size"] = stripslashes(get_option('WPPortfolio_setting_stw_thumb_size'));
    }
    
    // Try to grab the thumbnail
    $imagefile = WPPortfolio_getCachedThumbnail($url, 
    								$actualThumbPath, 
    								trailingslashit(trailingslashit(WP_PLUGIN_URL).WPP_THUMBNAIL_PATH),  
    								trailingslashit(trailingslashit(WP_PLUGIN_URL).WPP_THUMB_DEFAULTS), 
    								$args, 
    								$setting_cache_days,
    								false, 
    								$setting_fetch_method);    
        
    return $imagefile;
}


/**
 * Function that gets a list of the website groups and converts it into an HTML SELECT combobox.
 * @param $selectedid The currently selected group ID, '1' by default.
 * @return String The HTML containing a list of groups.
 */
function WPPortfolio_getListOfWebsiteGroups($selectedid = 1, $htmlname)
{
	global $wpdb;
	$table_name = $wpdb->prefix . TABLE_WEBSITE_GROUPS;
	$groups = $wpdb->get_results("SELECT * FROM $table_name ORDER BY groupname", OBJECT);
	
	$html = "";	
	
	if ($groups)
	{		
		// Start HTML SELECT code
		$html .= "<select name=\"$htmlname\" >\n";
				
		// Add all groups as an option
		foreach ($groups as $groupdetails)
		{
			// See if item is selected
			$selected = "";
			if ($groupdetails->groupid == $selectedid)
				$selected = " selected";
						
			// Add some whitespace to the string to pad it a little, avoids the dropdown box looking nasty on Windows
			$html .= '<option value="'.$groupdetails->groupid.'"'.$selected.'>'.stripslashes($groupdetails->groupname)."&nbsp;&nbsp;&nbsp;&nbsp;</option>\n";
		}
		
		// Finish select
		$html .= "</select>\n";
	}

	return $html;
}


/**
 * Function: WPPortfolio_showMessage();
 *
 * Shows settings saved message.
 */
function WPPortfolio_showMessage($message = "Settings saved.", $errormsg = false)
{
	if ($errormsg) {
		echo '<div id="message" class="error">';
	}
	else {
		echo '<div id="message" class="updated fade">';
	}

	echo "<p><strong>$message</strong></p></div>";
}

/**
 * Function: WPPortfolio_showRedirectionMessage();
 *
 * Shows settings saved and page being redirected message.
 */
function WPPortfolio_showRedirectionMessage($message, $target, $delay)
{
?>
	<div id="message" class="updated fade">
		<p>
			<strong><?php echo $message; ?><br /><br />
			Redirecting in <?php echo $delay ?> seconds. Please click <a href="<?php echo $target; ?>">here</a> if you do not wish to wait.</strong>
		</p>
	</div>
	
	<SCRIPT language="JavaScript">
    <!--
            function getgoing() {
                     top.location="<?php echo $target; ?>";
            }

            if (top.frames.length==0) {
                setTimeout('getgoing()',<?php echo $delay * 1000 ?>);
            }
	//-->
	</SCRIPT>
	<?php
}

/**
 * Determine if the specified key is valid, i.e. containing only letters and numbers.
 * @param $key The key to check
 * @return Boolean True if the key is valid, false otherwise.
 */
function WPPortfolio_isValidKey($key) 
{
	// Ensure the key only contains letters and numbers
	return preg_match('/^[a-z0-9A-Z]+$/', $key);
}

/**
 * Recursively delete a directory
 *
 * @param string $dir Directory name
 * @param boolean $deleteRootToo Delete specified top-level directory as well
 */
function WPPortfolio_unlinkRecursive($dir, $deleteRootToo)
{
    if(!$dh = @opendir($dir)) {
        return;
    }
    while (false !== ($obj = readdir($dh)))
    {
        if($obj == '.' || $obj == '..') {
            continue;
        }

        if (!@unlink($dir . '/' . $obj)) {
            unlinkRecursive($dir.'/'.$obj, true);
        }
    }

    closedir($dh);
   
    if ($deleteRootToo) {
        @rmdir($dir);
    }
   
    return;
} 

/**
 * Replace all occurances of the search string with the replacement. Uses alternative for str_ireplace if not available.
 * @param $searchstr The string to search for.
 * @param $replacestr The string to replace the search string with.
 * @param $haystack The string to search.
 * @return String The text with the replaced string.
 */
function WPPortfolio_replaceString($searchstr, $replacestr, $haystack) {
	
	// Faster, but in PHP5.
	if (function_exists("str_ireplace")) {
		return str_ireplace($searchstr, $replacestr, $haystack);
	}
	// Slower but handles PHP4
	else { 
		return preg_replace("/$searchstr/i", $replacestr, $haystack);
	}
}



/**
 * Get a thumbnail, caching it first if possible.
 * 
 * @param $url The URL to get a thumbnail for.
 * @param $actualThumbPath The actual path on the server to store the cached thumbnail.
 * @param $webFilePath The web URL path for the thumbnails that relates to the actual file path.
 * @param $pendingThumbPath The path for images when a thumbnail cannot be loaded.
 * @param $args The arguments that contain the access ID and size info to get the thumbnail.
 * @param $cachedays The number of days to cache thumbnails.
 * @param $forceUpdate If true, force the retrieval over the web of the thumbnail.
 * @param $fetchMethod The method used to fetch a thumbnail (either curl or fopen).
 * 
 * @return String The full path of the image to render.
 */
function WPPortfolio_getCachedThumbnail($url, $actualThumbPath, $webFilePath, $pendingThumbPath,
							$args = null, $cachedays = 7, $forceUpdate = false, $fetchMethod = "curl")
{
	// Don't try something if not enough arguements.
	if (!$args || !$url || !$actualThumbPath || !$webFilePath)
		return false;

	// Use arguements to work out the target filename
	$filename = md5($url.serialize($args)).".jpg";
	$filepath = trailingslashit($actualThumbPath).$filename;
	$size = $args['Size'];
	
	// Work out the date for expired thumbnails
	$cutoff = time() - 3600 * 24 * $cachedays;
		
	// Work out if we need to update the thumbnail
	$needupdate = ($forceUpdate || !file_exists($filepath) || filemtime($filepath) <= $cutoff);
	if ($needupdate)
	{
		// Try to download the file
		if ($jpgurl = WPPortfolio_checkWebsiteThumbnailCaptured($url, $args, $fetchMethod)) {		
			WPPortfolio_downloadRemoteImageToLocalPath($jpgurl, $filepath, $fetchMethod);
		}
	}

	// File downloaded successfully
	if (file_exists($filepath)) {
		return "$webFilePath/$filename";
	}
	// Something went wrong, so return default image 
	else {
		return "$pendingThumbPath$size.jpg";
	}
}

/**
 * Method that checks that the thumbnail for the specified website exists. 
 * @param $url The website to get the thumbnail for.
 * @param $args The arguements used for the HTTP request.
 * @param $fetchMethod The method used to fetch a thumbnail (either curl or fopen). 
 * 
 * @return The URL of the image file from the server if the request was successful, false otherwise.
 */
function WPPortfolio_checkWebsiteThumbnailCaptured($url, $args = null, $fetchMethod = "curl")
{
	$args = is_array($args) ? $args : array();	
	
	// Don't try something if not enough arguements.
	if (!$args || !$url)
		return false;
		
    $args["Service"] = "ShrinkWebUrlThumbnail";
    $args["Action"] = "Thumbnail";		
	$args["stwUrl"] = $url; // now url is added to the arguements
	
	$request_url = urldecode("http://www.shrinktheweb.com/xino.php?".http_build_query($args,'','&'));	// avoid &amp;
	
	// Check that the thumbnail exists, from the STW server
	// Use cURL if possible
	if ($fetchMethod == "curl" && function_exists('curl_init')) 
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $request_url);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER,1); // return into a variable
		$remotedata = curl_exec($ch);
		curl_close($ch);		
	}
	// This may be disabled as a security measure
	else
	{
		$remotedata = file_get_contents($request_url);
	}
	   
	$regex = '/<[^:]*:Thumbnail\\s*(?:Exists=\"((?:true)|(?:false))\")?[^>]*>([^<]*)<\//';
	if (preg_match($regex, $remotedata, $matches) == 1 && $matches[1] == "true")
		return $matches[2];
		
	// TODO Handle the case when there's an error.  That's when the preg_match fails.
	// else echo $line;

	return false;
}

/**
 * Method to get image at the specified remote URL and attempt to save it to the specifed local path.
 * @param $remoteURL The URL of the remote image to download.
 * @param $localPath The path to use to store the image locally.
 * @param $fetchMethod The method used to fetch a thumbnail (either curl or fopen). 
 */
function WPPortfolio_downloadRemoteImageToLocalPath($remoteURL, $localPath, $fetchMethod = "curl")
{
	if ($fetchMethod == "curl" && function_exists('curl_init'))
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $remoteURL);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER,1); // return into a variable
		$imagedata = $GetImage = curl_exec($ch);
		
		$localFileHandle = fopen($localPath, "w+");
		fputs($localFileHandle, $imagedata);
		fclose($localFileHandle);
	}
	else
	{		
		$imagedata = file_get_contents($remoteURL);
		$localFileHandle = fopen($localPath, "w+");
		fputs($localFileHandle, $imagedata);
		fclose($localFileHandle);
	}
}
?>